@@ -18,4 +18,7 @@ from .icarl_resnet import *
 from .ncm_classifier import NCMClassifier
 from .base_model import BaseModel
 from .helper_method import as_multitask
-from .pnn import PNN
+from .pnn import *
+from .generator import *
+from .lenet5 import LeNet5
+from .slim_resnet18 import SlimResNet18, MTSlimResNet18
